// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_vx_fs_fccv_h)
#define octave_vx_fs_fccv_h 1
#include "octave-config.h"
#include "fCColVector.h"
  extern OCTAVE_API FloatComplexColumnVector operator + (const float&, const FloatComplexColumnVector&);
  extern OCTAVE_API FloatComplexColumnVector operator - (const float&, const FloatComplexColumnVector&);
  extern OCTAVE_API FloatComplexColumnVector operator * (const float&, const FloatComplexColumnVector&);
  extern OCTAVE_API FloatComplexColumnVector operator / (const float&, const FloatComplexColumnVector&);
#endif
