// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_smx_sbm_bm_h)
#define octave_smx_sbm_bm_h 1
#include "octave-config.h"
#include "boolMatrix.h"
#include "boolSparse.h"
#include "Sparse-op-defs.h"
  extern OCTAVE_API SparseBoolMatrix mx_el_eq (const SparseBoolMatrix&, const boolMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_ne (const SparseBoolMatrix&, const boolMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_and (const SparseBoolMatrix&, const boolMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_or (const SparseBoolMatrix&, const boolMatrix&);
#endif
