// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_fdm_fcdm_h)
#define octave_mx_fdm_fcdm_h 1
#include "octave-config.h"
#include "fCDiagMatrix.h"
#include "fDiagMatrix.h"
  extern OCTAVE_API FloatComplexDiagMatrix operator + (const FloatDiagMatrix&, const FloatComplexDiagMatrix&);
  extern OCTAVE_API FloatComplexDiagMatrix operator - (const FloatDiagMatrix&, const FloatComplexDiagMatrix&);
  extern OCTAVE_API FloatComplexDiagMatrix product (const FloatDiagMatrix&, const FloatComplexDiagMatrix&);
#endif
