// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_cdm_m_h)
#define octave_mx_cdm_m_h 1
#include "octave-config.h"
#include "CMatrix.h"
#include "CDiagMatrix.h"
#include "dMatrix.h"
  extern OCTAVE_API ComplexMatrix operator + (const ComplexDiagMatrix&, const Matrix&);
  extern OCTAVE_API ComplexMatrix operator - (const ComplexDiagMatrix&, const Matrix&);
  extern OCTAVE_API ComplexMatrix operator * (const ComplexDiagMatrix&, const Matrix&);
#endif
