// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_s_i8nda_h)
#define octave_mx_s_i8nda_h 1
#include "octave-config.h"
#include "int8NDArray.h"
  extern OCTAVE_API int8NDArray operator + (const double&, const int8NDArray&);
  extern OCTAVE_API int8NDArray operator - (const double&, const int8NDArray&);
  extern OCTAVE_API int8NDArray operator * (const double&, const int8NDArray&);
  extern OCTAVE_API int8NDArray operator / (const double&, const int8NDArray&);
  extern OCTAVE_API boolNDArray mx_el_lt (const double&, const int8NDArray&);
  extern OCTAVE_API boolNDArray mx_el_le (const double&, const int8NDArray&);
  extern OCTAVE_API boolNDArray mx_el_ge (const double&, const int8NDArray&);
  extern OCTAVE_API boolNDArray mx_el_gt (const double&, const int8NDArray&);
  extern OCTAVE_API boolNDArray mx_el_eq (const double&, const int8NDArray&);
  extern OCTAVE_API boolNDArray mx_el_ne (const double&, const int8NDArray&);
  extern OCTAVE_API boolNDArray mx_el_and (const double&, const int8NDArray&);
  extern OCTAVE_API boolNDArray mx_el_or (const double&, const int8NDArray&);
  extern OCTAVE_API boolNDArray mx_el_and_not (const double&, const int8NDArray&);
  extern OCTAVE_API boolNDArray mx_el_or_not (const double&, const int8NDArray&);
#endif
