// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_smx_sm_scm_h)
#define octave_smx_sm_scm_h 1
#include "octave-config.h"
#include "CSparse.h"
#include "dSparse.h"
#include "Sparse-op-defs.h"
  extern OCTAVE_API SparseComplexMatrix operator + (const SparseMatrix&, const SparseComplexMatrix&);
  extern OCTAVE_API SparseComplexMatrix operator - (const SparseMatrix&, const SparseComplexMatrix&);
  extern OCTAVE_API SparseComplexMatrix product (const SparseMatrix&, const SparseComplexMatrix&);
  extern OCTAVE_API SparseComplexMatrix quotient (const SparseMatrix&, const SparseComplexMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_lt (const SparseMatrix&, const SparseComplexMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_le (const SparseMatrix&, const SparseComplexMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_ge (const SparseMatrix&, const SparseComplexMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_gt (const SparseMatrix&, const SparseComplexMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_eq (const SparseMatrix&, const SparseComplexMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_ne (const SparseMatrix&, const SparseComplexMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_and (const SparseMatrix&, const SparseComplexMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_or (const SparseMatrix&, const SparseComplexMatrix&);
#endif
