// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_fcdm_fs_h)
#define octave_mx_fcdm_fs_h 1
#include "octave-config.h"
#include "fCDiagMatrix.h"
  extern OCTAVE_API FloatComplexDiagMatrix operator * (const FloatComplexDiagMatrix&, const float&);
  extern OCTAVE_API FloatComplexDiagMatrix operator / (const FloatComplexDiagMatrix&, const float&);
#endif
