// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_vx_rv_cs_h)
#define octave_vx_rv_cs_h 1
#include "octave-config.h"
#include "CRowVector.h"
#include "dRowVector.h"
#include "oct-cmplx.h"
  extern OCTAVE_API ComplexRowVector operator + (const RowVector&, const Complex&);
  extern OCTAVE_API ComplexRowVector operator - (const RowVector&, const Complex&);
  extern OCTAVE_API ComplexRowVector operator * (const RowVector&, const Complex&);
  extern OCTAVE_API ComplexRowVector operator / (const RowVector&, const Complex&);
#endif
