// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_i64nda_ui16nda_h)
#define octave_mx_i64nda_ui16nda_h 1
#include "octave-config.h"
#include "int64NDArray.h"
#include "uint16NDArray.h"
  extern OCTAVE_API boolNDArray mx_el_lt (const int64NDArray&, const uint16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_le (const int64NDArray&, const uint16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_ge (const int64NDArray&, const uint16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_gt (const int64NDArray&, const uint16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_eq (const int64NDArray&, const uint16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_ne (const int64NDArray&, const uint16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_and (const int64NDArray&, const uint16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_or (const int64NDArray&, const uint16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_not_and (const int64NDArray&, const uint16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_not_or (const int64NDArray&, const uint16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_and_not (const int64NDArray&, const uint16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_or_not (const int64NDArray&, const uint16NDArray&);
#endif
