## Copyright (C) 2006-2019 John W. Eaton
##
## This file is part of Octave.
##
## Octave is free software: you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, see
## <https://www.gnu.org/licenses/>.

## -*- texinfo -*-
## @defvr {Automatic Variable} ans
## The most recently computed result that was not explicitly assigned to a
## variable.
##
## For example, after the expression
##
## @example
## 3^2 + 4^2
## @end example
##
## @noindent
## is evaluated, the value returned by @code{ans} is 25.
## @end defvr


## Mark file as being tested.  No real test needed for a documentation .m file
%!assert (1)
