// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_fnda_ui8_h)
#define octave_mx_fnda_ui8_h 1
#include "octave-config.h"
#include "uint8NDArray.h"
#include "fNDArray.h"
#include "oct-inttypes.h"
  extern OCTAVE_API uint8NDArray operator + (const FloatNDArray&, const octave_uint8&);
  extern OCTAVE_API uint8NDArray operator - (const FloatNDArray&, const octave_uint8&);
  extern OCTAVE_API uint8NDArray operator * (const FloatNDArray&, const octave_uint8&);
  extern OCTAVE_API uint8NDArray operator / (const FloatNDArray&, const octave_uint8&);
  extern OCTAVE_API boolNDArray mx_el_lt (const FloatNDArray&, const octave_uint8&);
  extern OCTAVE_API boolNDArray mx_el_le (const FloatNDArray&, const octave_uint8&);
  extern OCTAVE_API boolNDArray mx_el_ge (const FloatNDArray&, const octave_uint8&);
  extern OCTAVE_API boolNDArray mx_el_gt (const FloatNDArray&, const octave_uint8&);
  extern OCTAVE_API boolNDArray mx_el_eq (const FloatNDArray&, const octave_uint8&);
  extern OCTAVE_API boolNDArray mx_el_ne (const FloatNDArray&, const octave_uint8&);
  extern OCTAVE_API boolNDArray mx_el_and (const FloatNDArray&, const octave_uint8&);
  extern OCTAVE_API boolNDArray mx_el_or (const FloatNDArray&, const octave_uint8&);
  extern OCTAVE_API boolNDArray mx_el_not_and (const FloatNDArray&, const octave_uint8&);
  extern OCTAVE_API boolNDArray mx_el_not_or (const FloatNDArray&, const octave_uint8&);
#endif
