// DO NOT EDIT!  Generated automatically by mk-builtins.pl.

#if ! defined (octave_builtin_defun_decls_h)
#define octave_builtin_defun_decls_h 1

#include "octave-config.h"

#include "ovl.h"

namespace octave
{
  class interpreter;
}

extern OCTINTERP_API octave_value_list
Fdaspk_options (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdasrt_options (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdassl_options (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flsode_options (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fquad_options (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisguirunning (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fargv (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fprogram_invocation_name (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fprogram_name (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsparse_auto_mutate (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flogical (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fiscell (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcell (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fiscellstr (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcellstr (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstruct2cell (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fclass (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisa (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__parent_classes__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisobject (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fismethod (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__methods__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsuperiorto (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Finferiorto (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__meta_get_package__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__superclass_reference__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__meta_class_query__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmetaclass (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffunctions (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffunc2str (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstr2func (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fis_function_handle (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Finline (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fformula (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fargnames (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fvectorize (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__java_init__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__java_exit__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FjavaObject (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FjavaMethod (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__java_get__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__java_set__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__java2mat__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fjava_matrix_autoconversion (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fjava_unsigned_autoconversion (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdebug_java (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisjava (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisnull (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FonCleanup (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstruct (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisstruct (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__fieldnames__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisfield (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fnumfields (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcell2struct (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frmfield (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstruct_levels_to_print (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fprint_struct_array_contents (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftypeinfo (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__dump_typeinfo__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fnargin (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fnargout (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Foptimize_subsasgn_calls (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisargout (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdouble (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsingle (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fint8 (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fint16 (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fint32 (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fint64 (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fuint8 (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fuint16 (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fuint32 (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fuint64 (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsizeof (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsubsref (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsubsasgn (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fis_sq_string (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fis_dq_string (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdisable_permutation_matrix (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdisable_diagonal_matrix (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdisable_range (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fiskeyword (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fautoload (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmfilename (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsource (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffeval (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbuiltin (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Feval (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fassignin (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fevalin (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fevalc (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__parser_debug_flag__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__parse_file__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__profiler_enable__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__profiler_reset__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__profiler_data__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fend (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmax_recursion_depth (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fwhos_line_format (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsilent_functions (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstring_fill_char (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FPS4 (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fecho (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fjit_failcnt (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdebug_jit (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fjit_enable (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fjit_startcnt (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__betainc__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__contourc__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__dsearchn__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__expint__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__gammainc__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ichol0__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__icholt__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ilu0__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__iluc__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ilutp__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__lin_interpn__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__magick_read__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__magick_write__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__magick_ping__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__magick_finfo__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__magick_formats__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__pchip_deriv__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__qp__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbalance (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbesselj (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbessely (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbesseli (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbesselk (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbesselh (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fairy (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbitand (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbitor (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbitxor (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbitshift (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fflintmax (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fintmax (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fintmin (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsizemax (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbsxfun (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmax_stack_depth (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fwho (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fwhos (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcellfun (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Farrayfun (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fnum2cell (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmat2cell (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcellslices (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcellindexmat (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcolloc (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fconv2 (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fconvn (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdaspk (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdasrt (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdassl (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fall (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fany (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fatan2 (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fhypot (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flog2 (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frem (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmod (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcumprod (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcumsum (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdiag (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fprod (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fhorzcat (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fvertcat (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcat (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fpermute (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fipermute (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flength (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fndims (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fnumel (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsize (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsize_equal (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fnnz (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fnzmax (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frows (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcolumns (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsum (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsumsq (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fislogical (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisinteger (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fiscomplex (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisfloat (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcomplex (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisreal (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisempty (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisnumeric (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisscalar (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisvector (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisrow (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fiscolumn (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fismatrix (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fissquare (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fones (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fzeros (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FInf (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FNaN (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fe (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Feps (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fpi (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frealmax (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frealmin (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FI (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FNA (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffalse (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftrue (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Feye (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flinspace (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fresize (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Freshape (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fvec (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsqueeze (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffull (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fnorm (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fnot (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fuplus (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fuminus (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftranspose (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fctranspose (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fplus (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fminus (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmtimes (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmrdivide (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmpower (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmldivide (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flt (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fle (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Feq (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fge (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgt (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fne (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftimes (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frdivide (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fpower (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fldivide (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fand (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
For (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcolon (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftic (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftoc (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcputime (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsort (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__sort_rows_idx__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fissorted (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fnth_element (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__accumarray_sum__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__accumarray_min__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__accumarray_max__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__accumdim_sum__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmerge (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdiff (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frepelems (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbase64_encode (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbase64_decode (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdbstop (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdbclear (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdbstatus (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdbwhere (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdbtype (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdblist (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdbstack (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdbup (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdbdown (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdbstep (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdbcont (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdbquit (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisdebugmode (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__db_next_breakpoint_quiet__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FOCTAVE_HOME (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FOCTAVE_EXEC_HOME (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FOCTAVE_VERSION (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdet (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcd (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fpwd (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Freaddir (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__mkdir__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frmdir (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flink (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsymlink (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Freadlink (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frename (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fglob (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__wglob__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__fnmatch__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffilesep (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fpathsep (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fconfirm_recursive_rmdir (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fhave_window_system (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdlmread (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdot (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fblkmm (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Feig (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fellipj (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FEDITOR (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FEXEC_PATH (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FIMAGE_PATH (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frethrow (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ferror (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fwarning (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flasterror (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flasterr (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flastwarn (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbeep_on_error (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdebug_on_error (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdebug_on_warning (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffft (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fifft (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffft2 (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fifft2 (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffftn (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fifftn (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffclose (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffclear (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffflush (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffgetl (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffgets (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffskipl (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffopen (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffreport (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffrewind (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffseek (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fftell (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffprintf (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fprintf (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffputs (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fputs (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsprintf (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffscanf (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsscanf (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fscanf (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftextscan (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__textscan__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffread (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffwrite (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffeof (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fferror (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fpopen (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fpclose (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftempname (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftmpfile (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmkstemp (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fumask (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FP_tmpdir (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FSEEK_SET (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FSEEK_CUR (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FSEEK_END (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstdin (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstdout (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstderr (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffilter (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffind (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgcd (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgetgrent (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgetgrgid (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgetgrnam (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsetgrent (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fendgrent (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgetpwent (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgetpwuid (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgetpwnam (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsetpwent (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fendpwent (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgetrusage (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgivens (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__update_normals__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fishghandle (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__is_handle_visible__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Freset (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fset (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fget (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__get__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_figure__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__calc_dimensions__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_axes__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_line__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_text__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_image__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_surface__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_patch__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_light__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_hggroup__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_uimenu__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_uicontrol__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_uibuttongroup__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_uipanel__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_uicontextmenu__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_uitable__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_uitoolbar__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_uipushtool__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_uitoggletool__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_delete__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_handles__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_figure_handles__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_execute_callback__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__go_post_callback__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__image_pixel_size__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Favailable_graphics_toolkits (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fregister_graphics_toolkit (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Floaded_graphics_toolkits (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__show_figure__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdrawnow (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Faddlistener (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdellistener (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Faddproperty (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fwaitfor (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__zoom__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__get_frame__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgsvd (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fhash (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fget_help_text (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fget_help_text_from_file (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__operators__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__builtins__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flocalfunctions (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__which__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__list_functions__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbuilt_in_docstrings_file (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdoc_cache_file (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Finfo_file (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Finfo_program (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmakeinfo_program (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsuppress_verbose_help_message (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftexi_macros_file (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fhess (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fhex2num (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fnum2hex (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Finput (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fyes_or_no (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fkeyboard (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcompletion_matches (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Freadline_read_init_file (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Freadline_re_read_init_file (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fadd_input_event_hook (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fremove_input_event_hook (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FPS1 (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FPS2 (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcompletion_append_char (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__request_drawnow__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__gud_mode__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__mfile_encoding__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Finv (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__version_info__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fquit (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fatexit (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fkron (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgenpath (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frehash (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcommand_line_path (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frestoredefaultpath (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__pathorig__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fpath (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Faddpath (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frmpath (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__dump_load_path__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fload (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsave (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcrash_dumps_octave_core (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsave_default_options (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Foctave_core_file_limit (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Foctave_core_file_name (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Foctave_core_file_options (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsave_header_format_string (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flookup (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsave_precision (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flsode (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flu (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fluupdate (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fabs (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Facos (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Facosh (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fangle (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Farg (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fasin (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fasinh (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fatan (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fatanh (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcbrt (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fceil (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fconj (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcos (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcosh (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ferf (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ferfinv (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ferfcinv (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ferfc (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ferfcx (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ferfi (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdawson (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fexp (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fexpm1 (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisfinite (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffix (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffloor (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgamma (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fimag (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisalnum (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisalpha (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisascii (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fiscntrl (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisdigit (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisinf (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisgraph (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fislower (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisna (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisnan (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisprint (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fispunct (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisspace (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisupper (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisxdigit (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flgamma (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flog (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flog10 (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flog1p (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Freal (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fround (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Froundb (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsign (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsignbit (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsin (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsinh (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsqrt (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftan (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftanh (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftolower (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftoupper (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmatrix_type (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmin (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmax (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcummin (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcummax (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmgorth (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fnproc (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fedit_history (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fhistory (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frun_history (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fhistory_control (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fhistory_size (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fhistory_file (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fhistory_timestamp_format_string (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fhistory_save (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__octave_link_enabled__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__octave_link_edit_file__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__octave_link_question_dialog__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__octave_link_file_dialog__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__octave_link_list_dialog__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__octave_link_input_dialog__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__octave_link_named_icon__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__octave_link_show_preferences__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__octave_link_gui_preference__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__octave_link_file_remove__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__octave_link_file_renamed__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fopenvar (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__octave_link_show_doc__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__octave_link_register_doc__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__octave_link_unregister_doc__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fordschur (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdiary (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmore (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fterminal_size (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fpage_output_immediately (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fpage_screen_output (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FPAGER (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FPAGER_FLAGS (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fpinv (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Foutput_precision (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frats (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdisp (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffdisp (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdisplay (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fformat (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffixed_point_format (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fprint_empty_dimensions (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsplit_long_rows (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fpsi (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fquad (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fquadcc (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fqz (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frand (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frandn (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frande (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frandg (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frandp (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frandperm (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frcond (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fregexp (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fregexpi (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fregexprep (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fschur (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Frsf2csf (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__display_tokens__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__token_count__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__lexer_debug_flag__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FSIG (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdebug_on_interrupt (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsighup_dumps_octave_core (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsigquit_dumps_octave_core (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsigterm_dumps_octave_core (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fissparse (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsparse (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fspalloc (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fspparms (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsqrtm (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstrfind (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstrrep (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fchar (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstrvcat (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fischar (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstrcmp (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstrncmp (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstrcmpi (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstrncmpi (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstr2double (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__native2unicode__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__unicode2native__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Funicode_idx (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flist_in_columns (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsub2ind (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Find2sub (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsvd (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsvd_driver (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsylvester (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fignore_function_time_stamp (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__current_scope__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__dump_symtab_info__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__get_cmdline_fcn_txt__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdup2 (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fexec (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fpopen2 (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffcntl (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffork (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgetpgrp (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgetpid (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgetppid (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgetegid (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgetgid (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgeteuid (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgetuid (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fkill (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flstat (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmkfifo (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fpipe (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstat (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FS_ISREG (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FS_ISDIR (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FS_ISCHR (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FS_ISBLK (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FS_ISFIFO (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FS_ISLNK (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FS_ISSOCK (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgethostname (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Funame (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Funlink (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fwaitpid (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FWIFEXITED (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FWEXITSTATUS (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FWIFSIGNALED (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FWTERMSIG (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FWCOREDUMP (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FWIFSTOPPED (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FWSTOPSIG (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FWIFCONTINUED (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fcanonicalize_file_name (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FF_DUPFD (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FF_GETFD (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FF_GETFL (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FF_SETFD (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FF_SETFL (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FO_APPEND (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FO_ASYNC (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FO_CREAT (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FO_EXCL (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FO_NONBLOCK (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FO_RDONLY (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FO_RDWR (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FO_SYNC (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FO_TRUNC (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FO_WRONLY (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FWNOHANG (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FWUNTRACED (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
FWCONTINUE (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__open_with_system_app__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fclc (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgetenv (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsetenv (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Funsetenv (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fwinqueryreg (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fkbhit (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fpause (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisieee (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fnative_float_format (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftilde_expand (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fget_home_directory (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__blas_version__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__lapack_version__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftime (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fgmtime (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Flocaltime (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmktime (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstrftime (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fstrptime (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fwarranty (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fsystem (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__octave_config_info__ (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftril (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftriu (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftsearch (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ftypecast (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbitpack (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fbitunpack (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Furlwrite (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Furlread (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ftp__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ftp_pwd__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ftp_cwd__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ftp_dir__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ftp_ascii__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ftp_binary__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ftp_close__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ftp_mode__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ftp_delete__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ftp_rmdir__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ftp_mkdir__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ftp_rename__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ftp_mput__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__ftp_mget__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisvarname (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffile_in_loadpath (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ffile_in_path (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdo_string_escapes (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fundo_string_escapes (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fis_absolute_filename (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fis_rooted_relative_filename (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmake_absolute_filename (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fdir_in_loadpath (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ferrno (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Ferrno_list (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisindex (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisstudent (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fisglobal (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fexist (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmlock (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmunlock (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmislocked (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fclear (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmissing_function_hook (const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
F__varval__ (octave::interpreter&, const octave_value_list& = octave_value_list (), int = 0);
extern OCTINTERP_API octave_value_list
Fmissing_component_hook (const octave_value_list& = octave_value_list (), int = 0);
#endif
