// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_s_cm_h)
#define octave_mx_s_cm_h 1
#include "octave-config.h"
#include "CMatrix.h"
  extern OCTAVE_API ComplexMatrix operator + (const double&, const ComplexMatrix&);
  extern OCTAVE_API ComplexMatrix operator - (const double&, const ComplexMatrix&);
  extern OCTAVE_API ComplexMatrix operator * (const double&, const ComplexMatrix&);
  extern OCTAVE_API ComplexMatrix operator / (const double&, const ComplexMatrix&);
  extern OCTAVE_API boolMatrix mx_el_lt (const double&, const ComplexMatrix&);
  extern OCTAVE_API boolMatrix mx_el_le (const double&, const ComplexMatrix&);
  extern OCTAVE_API boolMatrix mx_el_ge (const double&, const ComplexMatrix&);
  extern OCTAVE_API boolMatrix mx_el_gt (const double&, const ComplexMatrix&);
  extern OCTAVE_API boolMatrix mx_el_eq (const double&, const ComplexMatrix&);
  extern OCTAVE_API boolMatrix mx_el_ne (const double&, const ComplexMatrix&);
  extern OCTAVE_API boolMatrix mx_el_and (const double&, const ComplexMatrix&);
  extern OCTAVE_API boolMatrix mx_el_or (const double&, const ComplexMatrix&);
#endif
