// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_vx_crv_s_h)
#define octave_vx_crv_s_h 1
#include "octave-config.h"
#include "CRowVector.h"
  extern OCTAVE_API ComplexRowVector operator + (const ComplexRowVector&, const double&);
  extern OCTAVE_API ComplexRowVector operator - (const ComplexRowVector&, const double&);
  extern OCTAVE_API ComplexRowVector operator * (const ComplexRowVector&, const double&);
  extern OCTAVE_API ComplexRowVector operator / (const ComplexRowVector&, const double&);
#endif
