// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_cs_dm_h)
#define octave_mx_cs_dm_h 1
#include "octave-config.h"
#include "CDiagMatrix.h"
#include "oct-cmplx.h"
#include "dDiagMatrix.h"
  extern OCTAVE_API ComplexDiagMatrix operator * (const Complex&, const DiagMatrix&);
#endif
