// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_fs_fcm_h)
#define octave_mx_fs_fcm_h 1
#include "octave-config.h"
#include "fCMatrix.h"
  extern OCTAVE_API FloatComplexMatrix operator + (const float&, const FloatComplexMatrix&);
  extern OCTAVE_API FloatComplexMatrix operator - (const float&, const FloatComplexMatrix&);
  extern OCTAVE_API FloatComplexMatrix operator * (const float&, const FloatComplexMatrix&);
  extern OCTAVE_API FloatComplexMatrix operator / (const float&, const FloatComplexMatrix&);
  extern OCTAVE_API boolMatrix mx_el_lt (const float&, const FloatComplexMatrix&);
  extern OCTAVE_API boolMatrix mx_el_le (const float&, const FloatComplexMatrix&);
  extern OCTAVE_API boolMatrix mx_el_ge (const float&, const FloatComplexMatrix&);
  extern OCTAVE_API boolMatrix mx_el_gt (const float&, const FloatComplexMatrix&);
  extern OCTAVE_API boolMatrix mx_el_eq (const float&, const FloatComplexMatrix&);
  extern OCTAVE_API boolMatrix mx_el_ne (const float&, const FloatComplexMatrix&);
  extern OCTAVE_API boolMatrix mx_el_and (const float&, const FloatComplexMatrix&);
  extern OCTAVE_API boolMatrix mx_el_or (const float&, const FloatComplexMatrix&);
#endif
