// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_pm_fcm_h)
#define octave_mx_pm_fcm_h 1
#include "octave-config.h"
#include "fCMatrix.h"
#include "PermMatrix.h"
  extern OCTAVE_API FloatComplexMatrix operator * (const PermMatrix&, const FloatComplexMatrix&);
#endif
