// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_fm_fdm_h)
#define octave_mx_fm_fdm_h 1
#include "octave-config.h"
#include "fMatrix.h"
#include "fDiagMatrix.h"
  extern OCTAVE_API FloatMatrix operator + (const FloatMatrix&, const FloatDiagMatrix&);
  extern OCTAVE_API FloatMatrix operator - (const FloatMatrix&, const FloatDiagMatrix&);
  extern OCTAVE_API FloatMatrix operator * (const FloatMatrix&, const FloatDiagMatrix&);
#endif
