// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_cdm_dm_h)
#define octave_mx_cdm_dm_h 1
#include "octave-config.h"
#include "CDiagMatrix.h"
#include "dDiagMatrix.h"
  extern OCTAVE_API ComplexDiagMatrix operator + (const ComplexDiagMatrix&, const DiagMatrix&);
  extern OCTAVE_API ComplexDiagMatrix operator - (const ComplexDiagMatrix&, const DiagMatrix&);
  extern OCTAVE_API ComplexDiagMatrix product (const ComplexDiagMatrix&, const DiagMatrix&);
#endif
