// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_vx_fccv_fs_h)
#define octave_vx_fccv_fs_h 1
#include "octave-config.h"
#include "fCColVector.h"
  extern OCTAVE_API FloatComplexColumnVector operator + (const FloatComplexColumnVector&, const float&);
  extern OCTAVE_API FloatComplexColumnVector operator - (const FloatComplexColumnVector&, const float&);
  extern OCTAVE_API FloatComplexColumnVector operator * (const FloatComplexColumnVector&, const float&);
  extern OCTAVE_API FloatComplexColumnVector operator / (const FloatComplexColumnVector&, const float&);
#endif
