// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_fnda_fcnda_h)
#define octave_mx_fnda_fcnda_h 1
#include "octave-config.h"
#include "fCNDArray.h"
#include "fNDArray.h"
  extern OCTAVE_API FloatComplexNDArray operator + (const FloatNDArray&, const FloatComplexNDArray&);
  extern OCTAVE_API FloatComplexNDArray operator - (const FloatNDArray&, const FloatComplexNDArray&);
  extern OCTAVE_API FloatComplexNDArray product (const FloatNDArray&, const FloatComplexNDArray&);
  extern OCTAVE_API FloatComplexNDArray quotient (const FloatNDArray&, const FloatComplexNDArray&);
  extern OCTAVE_API boolNDArray mx_el_lt (const FloatNDArray&, const FloatComplexNDArray&);
  extern OCTAVE_API boolNDArray mx_el_le (const FloatNDArray&, const FloatComplexNDArray&);
  extern OCTAVE_API boolNDArray mx_el_ge (const FloatNDArray&, const FloatComplexNDArray&);
  extern OCTAVE_API boolNDArray mx_el_gt (const FloatNDArray&, const FloatComplexNDArray&);
  extern OCTAVE_API boolNDArray mx_el_eq (const FloatNDArray&, const FloatComplexNDArray&);
  extern OCTAVE_API boolNDArray mx_el_ne (const FloatNDArray&, const FloatComplexNDArray&);
  extern OCTAVE_API boolNDArray mx_el_and (const FloatNDArray&, const FloatComplexNDArray&);
  extern OCTAVE_API boolNDArray mx_el_or (const FloatNDArray&, const FloatComplexNDArray&);
  extern OCTAVE_API boolNDArray mx_el_not_and (const FloatNDArray&, const FloatComplexNDArray&);
  extern OCTAVE_API boolNDArray mx_el_not_or (const FloatNDArray&, const FloatComplexNDArray&);
  extern OCTAVE_API boolNDArray mx_el_and_not (const FloatNDArray&, const FloatComplexNDArray&);
  extern OCTAVE_API boolNDArray mx_el_or_not (const FloatNDArray&, const FloatComplexNDArray&);
#endif
