// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_dm_m_h)
#define octave_mx_dm_m_h 1
#include "octave-config.h"
#include "dMatrix.h"
#include "dDiagMatrix.h"
  extern OCTAVE_API Matrix operator + (const DiagMatrix&, const Matrix&);
  extern OCTAVE_API Matrix operator - (const DiagMatrix&, const Matrix&);
  extern OCTAVE_API Matrix operator * (const DiagMatrix&, const Matrix&);
#endif
