// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_fcs_fnda_h)
#define octave_mx_fcs_fnda_h 1
#include "octave-config.h"
#include "fCNDArray.h"
#include "oct-cmplx.h"
#include "fNDArray.h"
  extern OCTAVE_API FloatComplexNDArray operator + (const FloatComplex&, const FloatNDArray&);
  extern OCTAVE_API FloatComplexNDArray operator - (const FloatComplex&, const FloatNDArray&);
  extern OCTAVE_API FloatComplexNDArray operator * (const FloatComplex&, const FloatNDArray&);
  extern OCTAVE_API FloatComplexNDArray operator / (const FloatComplex&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_lt (const FloatComplex&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_le (const FloatComplex&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_ge (const FloatComplex&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_gt (const FloatComplex&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_eq (const FloatComplex&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_ne (const FloatComplex&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_and (const FloatComplex&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_or (const FloatComplex&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_and_not (const FloatComplex&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_or_not (const FloatComplex&, const FloatNDArray&);
#endif
